
/**
 * Check status of async action
 * @param {string} wstrActionGuid - action`s identifier
 * @param {number} timeout - delay before next check
 * @param {Connections} connection
 * @return {Promise.<Object>} - info about finished or failed action
 */
module.exports = async function asyncActionStateChecker ({
  wstrActionGuid,
  timeout = 0
}, connection) {
  const { openAPIClient } = require('@kl/openapi-client')
  return new Promise(function (resolve, reject) {
    setTimeout(async () => {
      try {
        const { result } = await openAPIClient
          .AsyncActionStateChecker
          .CheckActionState({
            wstrActionGuid
          }, connection)
        if (result.bFinalized === true) {
          if (result.bSuccededFinalized === true) {
            resolve({ result })
          } else {
            reject(result)
          }
          return
        }
        const innerCallResult = await asyncActionStateChecker({
          wstrActionGuid,
          timeout: result.lNextCheckDelay
        }, connection)
        resolve(innerCallResult)
      } catch (error) {
        reject(error)
      }
    }, timeout)
  })
}
