const BaseOpenAPIClient = require('../base-open-api-client')

/**
 * Аутентификация пользователя
 */
module.exports = class Auth extends BaseOpenAPIClient {

  constructor (...args) {
    super(...args)
    this.isPublicInterface = false
  }

  /**
   * Аутентификация пользователя на Open API сервере
   * @param {string} login - логин внутреннего KSC или доменного пользователя
   * @param {string} password - пароль
   * @param {string} domain - имя виртуального сервера
   * @param {boolean} isInternalUser - флаг, является ли пользователь внутрениим или доменным
   * @param {Connection} connection - TCP connection with Open API server
   * @param {string} wstrAuthKey - one-time authentication key, special type of authentication used to create gateway connections
   * @param {string} token - authorization token, generated to temporarily grant access to KSC without exposing credentials
   * @return {Promise}
   */
  login ({login, password, domain = '', isInternalUser = 0, wstrAuthKey, token, headers = {}}, connection) {
    if (!~[0, 1, '0', '1'].indexOf(isInternalUser)) {
      throw new Error('Flag "isInternalUser" value is mismatch.')
    }

    const authHeaders = wstrAuthKey
      ? {'Authorization': `KSCGW ${wstrAuthKey}`}
      : token
      ? {'Authorization': `KSCT ${token}`}
      : {'Authorization': `KSCBasic user="${base64Enconde(login)}", pass="${base64Enconde(password)}", domain="${base64Enconde(domain)}", internal="${isInternalUser}"`}

    Object.assign(authHeaders, headers)

    return this.baseRequest({command: 'login', headers: authHeaders, connection})
  }
}

/**
 * Encode string to base64
 * @param {string} string - string to encode
 * @return {string}
 */
function base64Enconde (str) {
  var encodedString = new Buffer(str).toString('base64')
  return encodedString
}
