const baseOpenAPIClient = require('../base-open-api-client')
const className = 'EventProcessingFactory'

module.exports = class EventProcessingFactory extends baseOpenAPIClient {
  /**
   * Create event processing iterator
   * @param {Array.<String>} vecFieldsToReturn - array of attribute names to return
   * @param {Array.<Object>} vecFieldsToOrder - array of containers with sorting params
   * @param {number} lifetimeSec - max result-set lifetime in seconds
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  CreateEventProcessing ({ vecFieldsToReturn, vecFieldsToOrder = [], lifetimeSec = 1000 } = {}, connection) {
    const options = {
      command: `${className}.CreateEventProcessing`,
      data: {
        vecFieldsToReturn, vecFieldsToOrder, lifetimeSec
      },
      connection
    }
    return this.baseRequest(options)
      .then(body => this.getResult(body, 'strIteratorId'))
  }

  /**
   * Create event processing iterator
   * @param {Object} pFilter - object containing values for attributes to filter events
   * @param {Array.<String>} vecFieldsToReturn - array of attribute names to return
   * @param {Array.<Object>} vecFieldsToOrder - array of containers with sorting params
   * @param {number} lifetimeSec - max result-set lifetime in seconds
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  CreateEventProcessing2 ({ pFilter, vecFieldsToReturn, vecFieldsToOrder = [], lifetimeSec = 1000 } = {}, connection) {
    const options = {
      command: `${className}.CreateEventProcessing2`,
      data: {
        pFilter,
        vecFieldsToReturn,
        vecFieldsToOrder,
        lifetimeSec
      },
      connection
    }
    return this.baseRequest(options)
      .then(body => this.getResult(body, 'strIteratorId'))
  }

  /**
   * Создание итератора для обработки событий хоста
   * @param {string} strDomainName - домен устройства (KLHST_WKS_WINDOMAIN)
   * @param {string} strHostName - имя устройства (KLHST_WKS_HOSTNAME)
   * @param {string} strProduct - идентификатор продукта
   * @param {string} strVersion - версия продукта
   * @param {Object} pFilter - фильтры
   * @param {number} lifetimeSec - длительность сессии
   * @param {Connection} connection
   * @return {Promise.<{result}|*>}
   * @constructor
   */
  async CreateEventProcessingForHost2 ({ strDomainName, strHostName, strProduct, strVersion, pFilter = {}, vecFieldsToReturn, vecFieldsToOrder = [], lifetimeSec = 1000 }, connection) {
    const options = {
      command: `${className}.CreateEventProcessingForHost2`,
      data: {
        strDomainName,
        strHostName,
        strProduct,
        strVersion,
        pFilter,
        lifetimeSec,
        vecFieldsToReturn,
        vecFieldsToOrder
      },
      connection
    }
    const body = await this.baseRequest(options)
    return this.getResult(body, 'strIteratorId')
  }
}
