const baseIterator = require('../base-iterator')
const className = 'EventProcessing'

module.exports = class EventProcessing extends baseIterator {
  /**
   * Get record count
   * @param {string} strIteratorId - iterator identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetRecordCount ({ strIteratorId }, connection) {
    return this.baseRequest({ command: `${className}.GetRecordCount`, data: { strIteratorId }, connection })
      .then(this.getResult)
  }

  /**
   * Returns elements contained in the specified result-set in the diapason from position nStart to position nEnd
   * @param {string} strIteratorId - iterator identifier
   * @param {number} nStart - start position
   * @param {number} nEnd - end position
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetRecordRange ({ strIteratorId, nStart, nEnd }, connection) {
    return this.baseRequest({ command: `${className}.GetRecordRange`, data: { strIteratorId, nStart, nEnd }, connection })
      .then(body => this.deserialize(body, 'pParamsEvents'))
  }

  /**
   *
   * @param {string} strIteratorId - iterator identifier
   * @param {array} events - contains range blocks to mass delete events
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   * @constructor
   */
  InitiateDelete ({ strIteratorId, pSettings }, connection) {
    return this.baseRequest({ command: `${className}.InitiateDelete`, data: { strIteratorId, pSettings: this.serialize(pSettings) }, connection })
      .then(body => this.deserialize(body, 'pParamsEvents'))
  }

  /**
   * Закрытие итератора
   * @param {string} strIteratorId - идентификатор итератора
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  async ReleaseIterator ({ strIteratorId }, connection) {
    const result = await this.baseRequest({
      command: `${className}.ReleaseIterator`,
      data: {
        strIteratorId
      },
      connection
    })
    return this.getBody(result)
  }

  /**
   * Получение (с помощью итератора) событий, опубликованных на конкретнеом хосте
   * @param {string} strDomainName - домен устройства (KLHST_WKS_WINDOMAIN)
   * @param {string} strHostName - имя устройства (KLHST_WKS_HOSTNAME)
   * @param {string} strProduct - идентификатор продукта
   * @param {string} strVersion - версия продукта
   * @param {Array.string} vecFieldsToReturn - список требуемых полей (List of event attributes)
   * @param {Connection} connection
   * @return {Promise.<*>}
   */
  async fetchEventsForHost ({
                 iteratorParams: {
                   strDomainName,
                   strHostName,
                   strProduct,
                   strVersion,
                   vecFieldsToReturn
                 }
               }, connection) {
    const openAPIClient = require('../open-api-client')
    const fetchFunction = this.getFetchInstance({
      iteratorIdPropertyName: 'strIteratorId',
      resetIterator: openAPIClient.EventProcessingFactory.CreateEventProcessingForHost2.bind(this),
      getRecordCount: this.GetRecordCount.bind(this),
      getRecordRange: this.GetRecordRange.bind(this),
      releaseIterator: this.ReleaseIterator.bind(this)
    })
    const result = await fetchFunction({
      iteratorParams: {
        strDomainName,
        strHostName,
        strProduct,
        strVersion,
        vecFieldsToReturn
      }
    }, connection)
    return result
  }
}
