const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'ExtAud'

/**
 * Provide an interface for working with ExtAudit subsystem.
 * This interface allow you to get a revision of an object and update description.
 */
module.exports = class ExtAud extends BaseOpenAPIClient {
  /**
   * @description Get object revision.
   * @param {longInt} nObjId - object id
   * @param {number} nObjType - one of object types
   * possible object types:
   * 0 - None
   * 1 - Policy
   * 2 - Task
   * 3 - Package
   * 4 - Server
   * 5 - Virtual server
   * 6 - User
   * 7 - Security Group
   * 8 - Administration Group
   * @param {number} nObjRevision - object revision
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetRevision ({ nObjId, nObjType, nObjRevision } = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetRevision`,
      data: { nObjId: 'long' + nObjId, nObjType, nObjRevision },
      connection
    })
      .then(body => this.getResult(body, 'pObjParams'))
  }
  /**
   * @description Update revision description.
   * @param {longInt} nObjId - object id
   * @param {number} nObjType - one of object types
   * possible object types:
   * 0 - None
   * 1 - Policy
   * 2 - Task
   * 3 - Package
   * 4 - Server
   * 5 - Virtual server
   * 6 - User
   * 7 - Security Group
   * 8 - Administration Group
   * @param {number} nObjRevision - object revision
   * @param {string} wstrNewDescription - new description of revision
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateRevisionDesc ({ nObjId, nObjType, nObjRevision, wstrNewDescription } = {}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateRevisionDesc`,
      data: { nObjId: 'long' + nObjId, nObjType, nObjRevision, wstrNewDescription },
      connection
    })
      .then(this.getResult)
  }

  /**
  * @description Final delete for deleted objects.
  * @param {arrObjects} nObjId - Array of pairs ObjId-ObjType. Max size of array is 100 elements.
  * @param {Connection} connection
  * @return {Promise.<TResult>}
  */
 FinalDelete ({ arrObjects } = {}, connection) {
   return this.baseRequest({
     command: `${className}.FinalDelete`,
     data: { arrObjects },
     connection
   }).then(this.getResult)
 }
}
