const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'FileCategorizer2'

module.exports = class FileCategorizer2 extends BaseOpenAPIClient {

    /**
     * Returns an object with category attributes like name, description, last update
    * 
    * @param {any} nCategoryId - id of category, not guid, ex.: 1
    * @param {any} connection 
    * @returns {pCategory:{CategoryType, fromMaster, inclusions:[], lastUpdateTime, name, uuid, version}}
    */
    async GetCategory (nCategoryId, connection) {
        const body = await this.baseRequest({
        command: `${className}.GetCategory`,
        data: {
            nCategoryId
        },
        connection
        })
        return this.getBody(body)
    }

    /**
     * Returns array of policies with references to specified category.
     * 
     * @param {any} nCatId  - id of category, not guid, ex.: 1
     * @param {any} connection 
     * @returns Policies array
     */
    async GetRefPolicies (nCatId, connection) {
        const body = await this.baseRequest({
        command: `${className}.GetRefPolicies`,
        data: {
            nCatId
        },
        connection
        })
        return this.getBody(body)
    }

     /**
     * Updating category whith specified ID
     * 
     * @param {int} nCategoryId Category ID which should be updated
     * @param {obj} pCategory Params object, can contain only fields that to be updated, ex: {name: 'Foo'}
     * @param {any} connection 
     * @returns pCategory New category body 
     */
    async UpdateCategory ({nCategoryId, pCategory}, connection) {
        const body = await this.baseRequest({
            command: `${className}.UpdateCategory`,
            data: {
                nCategoryId,
                pCategory
            },
            connection
        })
        return this.getBody(body)
    }	

    /**
     * Create category (simple, autoupdate or silverimage)
     * 
     * @param {obj} pCategory Params object, Category body 
     * @param {any} connection 
     * @returns Category id
     */
    async CreateCategory ({pCategory}, connection) {
        const body = await this.baseRequest({
            command: `${className}.CreateCategory`,
            data: {
                pCategory
            },
            connection
        })
        return this.deserialize(body, 'PxgRetVal')
    }	


    /**
     * Deleting category with specified nCategoryId
     * 
     * @param {any} nCategoryId 
     * @param {any} connection 
     * @returns 
     */
    async DeleteCategory (nCategoryId, connection) {
        const body = await this.baseRequest({
            command: `${className}.DeleteCategory`,
            data: {
                nCategoryId
            },
            connection
        })
        return this.getBody(body)
    }

    
    /**
     * Forces the update Force category update (for autoupdate and silverimage)
     * @param {int} nCategoryId Category id
     * @param {any} connection 
     * @returns response
     */
    async ForceCategoryUpdate (nCategoryId, connection) {
        const body = await this.baseRequest({
            command: `${className}.ForceCategoryUpdate`,
            data: {
                nCategoryId   
            },
            connection
        })
        return this.getBody(body)
    }

    /**
     * Initializing url for further file upload by putMethod
     * @param {*} args empty args
     * @param {*} connection 
     * @returns {string} URL for uploading
     */
    async InitFileUpload (args, connection) {
        const body = await this.baseRequest({
            command: `${className}.InitFileUpload`,
            connection
        })
        return this.getBody(body)
    }
    /**
     * 
     * Cancelling created url for file upload
     * @param {any} args 
     * @param {any} connection 
     * @returns empy
     */
    async CancelFileUpload (args, connection) {
        const body = await this.baseRequest({
            command: `${className}.CancelFileUpload`,
            connection
        })
        return this.getBody(body)
    }

    /**
     * Get files metadata from zip-archive.
     * To get result use AsyncActionStateChecker.CheckActionState.
     * @param {int} nCategoryId Category id
     * @param {any} connection 
     * @returns response
     */
    async GetFilesMetadata (ulFlag, connection) {
        const body = await this.baseRequest({
            command: `${className}.GetFilesMetadata`,
            data: {
                ulFlag   
            },
            connection
        })
        return this.getBody(body)
    }

    /**
     * Get file metadata.
     * To get result use AsyncActionStateChecker.CheckActionState.
     * @param {int} nCategoryId Category id
     * @param {any} connection 
     * @returns response
     */
    async GetFileMetadata (ulFlag, connection) {
        const body = await this.baseRequest({
            command: `${className}.GetFileMetadata`,
            data: {
                ulFlag   
            },
            connection
        })
        return this.getBody(body)
    }

    /**
     * Get files metadata from MSI.
     * To get result use AsyncActionStateChecker.CheckActionState.
     * @param {int} nCategoryId Category id
     * @param {any} connection 
     * @returns response
     */
    async GetFilesMetadataFromMSI (ulFlag, connection) {
        const body = await this.baseRequest({
            command: `${className}.GetFilesMetadataFromMSI`,
            data: {
                ulFlag   
            },
            connection
        })
        return this.getBody(body)
    }





}
