const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'InventoryApi'

module.exports = class InventoryApi extends BaseOpenAPIClient {
  GetSrvCompetitorIniFileInfoList ({ wstrType } = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetSrvCompetitorIniFileInfoList`,
      data: {
        wstrType
      },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Acquire all software applications
   *
   * @param { Object } pParams - Reserved but currently emprty properties container
   * @param { Object } connection
   */
  GetInvProductsList (pParams, connection) {
    return this.baseRequest({
      command: `${className}.GetInvProductsList`,
      data: {
        pParams: null
      },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Acquire all software application updates
   *
   * @param { Object } pParams - Reserved but currently emprty properties container
   * @param { Object } connection
   */
  GetInvPatchesList (pParams, connection) {
    return this.baseRequest({
      command: `${className}.GetInvPatchesList`,
      data: {
        pParams: null
      },
      connection
    })
      .then(this.deserialize)
  }

  /** Acquire list of observed applications
   * @param {Object} params - method parameters
   * @param {Object} connection - OpenAPI connection
   * @return {Array.<string>} pAppIds - collection of application `ProductID` */
  GetObservedApps (params, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.GetObservedApps`,
      connection,
      data: { pParams: null }
    }).then(this.deserialize)
  }

  /** Set list of observed applications
   * @param {Object} params - method parameters
   * @param {Array.<string>} params.pAppIds - collection of application `ProductID`
   * @param {Object} connection - OpenAPI connection
   * @return {void} */
  SetObservedApps (params, connection) {
    const { pAppIds } = params
    return this.baseRequest({
      command: `${this.constructor.name}.SetObservedApps`,
      connection,
      data: { pAppIds, pParams: null }
    }).then(this.deserialize)
  }

  /** Remove from database info about software applications which aren't installed on any host
   * @param {Object} params - method parameters
   * @param {Object} connection - OpenAPI connection
   * @return {void} */
  DeleteUninstalledApps (params, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.DeleteUninstalledApps`,
      connection,
      data: { pParams: null }
    }).then(this.deserialize)
  }

  /**
   * Acquire software applications which are installed on specified host.
   *
   * @param { string } param.szwHostId host name, a unique server-generated string (see KLHST_WKS_HOSTNAME attribute). It is NOT the same as computer network name (DNS-, FQDN-, NetBIOS-name)
   * @param { object } connection
   */
  GetHostInvProducts ({szwHostId}, connection) {
    return this.baseRequest({
      command: `${className}.GetHostInvProducts`,
      data: {
        pParams: null,
        szwHostId
      },
      connection
    }).then(this.deserialize)
  }
}
