const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'NlaDefinedNetworks'

module.exports = class NlaDefinedNetworks extends BaseOpenAPIClient {

  /**
   * Add nla-defined network
   * @param wstrNetworkName - Network must not exist
   * @returns {int} - newly created network id
   */
  async AddNetwork ({ wstrNetworkName }, connection) {
    const body = await this.baseRequest({
      command: `${className}.AddNetwork`,
      data: {
        wstrNetworkName
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Delete NLA-defined network
   * @param {int} nNetworkId - id of network to delete
   */
  async DeleteNetwork ({ nNetworkId }, connection) {
    const body = await this.baseRequest({
      command: `${className}.DeleteNetwork`,
      data: {
        nNetworkId
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Get NLA-defined network info
   * @param {int} nNetworkId - id of network
   * @returns {object} pNetwork - NLA-defined network
   */
  async GetNetworkInfo ({ nNetworkId }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetNetworkInfo`,
      data: {
        nNetworkId
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Get list of all NLA-defined networks
   * @returns {Array<object>} - Array of NLA-defined networks
   */
  async GetNetworksList (data, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetNetworksList`,
      connection
    })
    return this.getBody(body)
  }

  /**
   * Change nla-defined network
   * @param pNetwork - NLA-defined network
   */
  async SetNetworkInfo ({ pNetwork }, connection) {
    const body = await this.baseRequest({
      command: `${className}.SetNetworkInfo`,
      data: {
        pNetwork: this.serialize(pNetwork)
      },
      connection
    })
    return this.getBody(body)
  }
}
