const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'QBTNetworkListApi'

module.exports = class QBTNetworkListApi extends BaseOpenAPIClient {

    /**
     * Initiate action under the specified files from specified network list.
     * 
     * @param {string} listName - name of the network list.
     * @param {array} pItemsIds - collection of the file identifiers. Each entry of the array is {int} identifier of the file. (see "nId" attribute from Quarantine, Backup, TIF lists)
     * @param {string} taskName - name of the action ["RestoreObj"	Restore object from quarantine/backup, "DeleteObj"	Delete object from quarantine/backup, "ScanQuarantine"	Scan object on quarantine/tif]
     * @param {any} connection 
     * @deprecated pTaskParams	(params). Deprecated. May be empty but should be not null
     * @returns 
     */
    async AddListItemsTask ({listName, pItemsIds, taskName}, connection) {
        const body = await this.baseRequest({
            command: `${className}.AddListItemsTask`,
            data: {
                listName,
                pItemsIds,
                taskName,
                pTaskParams: null
            },
            connection
        })
        return this.getBody(body)
    }

    /**
     * Initiate action under the specified file from specified network list.
     * 
     * @param {string} listName - name of the network list.
     * @param {int} itemId - collection of the file identifiers. Each entry of the array is {int} identifier of the file. (see "nId" attribute from Quarantine, Backup, TIF lists)
     * @param {string} taskName - name of the action ["RestoreObj"	Restore object from quarantine/backup, "DeleteObj"	Delete object from quarantine/backup, "ScanQuarantine"	Scan object on quarantine/tif]
     * @param {any} connection
     * @deprecated pTaskParams	(params). Deprecated. May be empty but should be not null 
     * @returns 
     */
    async AddListItemTask ({listName, itemId, taskName}, connection) {
        const body = await this.baseRequest({
            command: `${className}.AddListItemTask`,
            data: {
                listName,
                itemId,
                taskName,
                pTaskParams: null
            },
            connection
        })
        return this.getBody(body)
    }

    /**
     * Acquire info about specified file from specified network list.
     * 
     * @param {any} itemId - identifier of the file. (see "nId" attribute from Quarantine, Backup, TIF lists)
     * @param {any} connection 
     * @returns 
     */
    async GetListItemInfo ({itemId}, connection) {
        const body = await this.baseRequest({
            command: `${className}.GetListItemInfo`,
            data: {
                itemId
            },
            connection
        })
        return this.getBody(body)
    }

}
