const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'UaControl'

module.exports = class UaControl extends BaseOpenAPIClient {
  /**
   * GetUpdateAgentsDisplayInfoForHost
   * @param {string} wstrHostId - Host id to find UAs
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetUpdateAgentsDisplayInfoForHost ({ wstrHostId }, connection) {
    return this.baseRequest({
      command: `${className}.GetUpdateAgentsDisplayInfoForHost`,
      data: { wstrHostId },
      connection
    })
      .then(body => this.getResult(body, 'pUaInfo'))
  }

  GetAssignUasAutomatically (data = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetAssignUasAutomatically`,
      connection
    })
      .then(this.getResult)
  }

  GetDefaultUpdateAgentRegistrationInfo (data = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetDefaultUpdateAgentRegistrationInfo`,
      connection
    })
      .then(body => this.getResult(body, 'pUaInfo'))
  }

  GetUpdateAgentsList (data = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetUpdateAgentsList`,
      connection
    })
      .then(body => this.deserialize(body, 'pUasArr'))
  }

  ModifyUpdateAgent ({ pUaInfo }, connection) {
    return this.baseRequest({
      command: `${className}.ModifyUpdateAgent`,
      data: {
        pUaInfo: this.serialize(pUaInfo)
      },
      connection
    })
      .then(this.getResult)
  }

  GetUpdateAgentInfo ({ wstrUaHostId }, connection) {
    return this.baseRequest({
      command: `${className}.GetUpdateAgentInfo`,
      data: {
        wstrUaHostId
      },
      connection
    })
      .then(body => this.getResult(body, 'pUaInfo'))
  }

  RegisterDmzGateway ({ pCgInfo }, connection) {
    return this.baseRequest({
      command: `${className}.RegisterDmzGateway`,
      data: {
        pCgInfo
      },
      connection
    })
      .then(this.getResult)
  }

  RegisterUpdateAgent ({ pUaInfo }, connection) {
    return this.baseRequest({
      command: `${className}.RegisterUpdateAgent`,
      data: {
        pUaInfo: this.serialize(pUaInfo)
      },
      connection
    })
      .then(this.getResult)
  }

  SetAssignUasAutomatically ({ bEnabled }, connection) {
    return this.baseRequest({
      command: `${className}.SetAssignUasAutomatically`,
      data: {
        bEnabled
      },
      connection
    })
      .then(this.getResult)
  }

  UnregisterUpdateAgent ({ wstrUaHostId }, connection) {
    return this.baseRequest({
      command: `${className}.UnregisterUpdateAgent`,
      data: {
        wstrUaHostId
      },
      connection
    })
      .then(this.getResult)
  }
}
