const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'UpdComps'

module.exports = class UpdComps extends BaseOpenAPIClient {

  /**
   * Requests asynchronous bases update
   * @param pParams -  Update operation parameters
   * @param wsRequestId - Unique user-generated request identifier of new asynchronous update operation
   * @param connection
   * @returns {Promise.<TResult>}
   * @constructor
   */
  UpdateAsync ({ pParams, wsRequestId }, connection) {
    return this.baseRequest({ command: `${className}.UpdateAsync`, data: { pParams, wsRequestId }, connection })
    .then(this.getResult)
  }

  /**
   * Stops operation by request
   * @param wsRequestId - Unique user-generated request identifier of new asynchronous update operation
   * @param bWait - Whether to wait for actual operation stop in blocking way (true) or just initiate asynchronous stop and return back ASAP (false)
   * @param connection
   * @returns {Promise.<TResult>}
   * @constructor
   */
  Stop ({ wsRequestId, bWait }, connection) {
    return this.baseRequest({ command: `${className}.Stop`, data: { wsRequestId, bWait }, connection })
    .then(this.getResult)
  }
}
