"use strict";

require("core-js/modules/es.array.concat");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.function.name");

require("core-js/modules/es.object.to-string");

require("core-js/modules/es.object.values");

require("core-js/modules/es.promise");

require("core-js/modules/es.regexp.constructor");

require("core-js/modules/es.regexp.to-string");

require("core-js/modules/es.string.iterator");

require("core-js/modules/web.dom-collections.for-each");

require("core-js/modules/web.dom-collections.iterator");

/* global Router */
var ControllerPrototype = require('./controller-prototype');

var router;
var routes = [];
var routeControllers = {};
var currentRoute = {};
var nextRoute = {};
var inTransit = false;
var prevRoute = {};
var isNavigationDisabled = false;
var onDisabledNavigation;
runtime.on('router-outlets::state-content-mounted', executeRouteCallbacks);

function executeRouteCallbacks() {
  if (!inTransit) {
    if (typeof currentRoute.config.callback === 'function') {
      if (currentRoute.config.resolve) {
        currentRoute.config.resolve().then(function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          currentRoute.config.callback(currentRoute.params, args);
        });
      } else {
        currentRoute.config.callback(currentRoute.params);
      }
    }

    if (Array.isArray(currentRoute.controllers)) {
      Promise.all(currentRoute.controllers).then(function (_controllers) {
        _controllers.forEach(function (_controller) {
          if (_controller.activate) {
            _controller.activate(currentRoute.params, currentRoute);
          } else if (_controller.onActivate) {
            _controller.onActivate(currentRoute.params, currentRoute);
          }
        });
      });
    }
  }
}

module.exports = function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$logger = _ref.logger,
      logger = _ref$logger === void 0 ? console : _ref$logger;

  function onNavigate(config, controllers, params) {
    prevRoute = currentRoute;
    currentRoute = {
      config: config,
      controllers: controllers,
      params: params
    };
    runtime.trigger('router::onNavigate', currentRoute);

    if (config.updateOutletAccurate) {
      _.forEach(config.outlets, function (outlet, outletName) {
        runtime.trigger("router-outlets::refresh:".concat(outletName));
      });
    } else {
      runtime.trigger('router-outlets::refresh');
    }

    var _tempControllers = controllers;
    currentRoute.config = _.assign({}, config, {
      onunload: function onunload() {
        deactivate(_tempControllers, currentRoute.params, config.onunload);
      }
    });
    inTransit = false;

    if (!config.outlets['state-content']) {
      executeRouteCallbacks();
    }
  }

  function deactivate(controllers, currentRouteParams, defaultOnunload) {
    if (Array.isArray(controllers)) {
      Promise.all(controllers).then(function (_controllers) {
        _controllers.forEach(function (_controller) {
          if (_controller.deactivate) {
            _controller.deactivate(currentRouteParams);
          } else if (_controller.onDeactivate) {
            _controller.onDeactivate(currentRouteParams);
          }
        });
      });
    }

    if (typeof defaultOnunload === 'function') {
      defaultOnunload();
    }
  }

  function validateRouteConfig(path, config) {
    var err = [];
    if (!config.layout) err.push('Layout should be specified. Example: {layout:app-root-nav-level}');

    if (config.sideComponent && (!Array.isArray(config.sideComponent.items) || !config.sideComponent.active)) {
      err.push('sideComponent config is invalid. "items" should be an array and "active" should be a string');
    }

    if (!config.stateName) err.push('State name is required.Example: {stateName:"users"}');
    if (config.resolve && typeof config.resolve !== 'function') err.push('{resolve} must be a function.');
    if (config.controller && !Array.isArray(config.controller)) err.push('Parameter "controller" must be an array.');

    var checkRegisterPath = function checkRegisterPath(path) {
      /* Start with: a-zA-Z0-9_
        then N times: a-zA-Z0-9_-
        dont end with: -
      */
      var allowedString = /\w[\w-]*(?<!-)/; // use for check dynamic paths. For example ":name"

      /* Start with: a-zA-Z_
        then N times: a-zA-Z0-9_
      */

      var allowedVariableName = /[a-zA-Z_]{1}[\w]*/;
      /* (0 or more)
        Start with: &
        then: "allowedString"
      */

      var urlParamMoreThan2 = new RegExp("(?:&".concat(allowedString.source, ")*"));
      /* (0 or 1)
        Start with: ?
        then: "allowedString"
        then: "urlParamMoreThan2"
      */

      var urlParams = new RegExp("(?:\\?".concat(allowedString.source).concat(urlParamMoreThan2.source, ")?"));
      /* (0 or 1)
        start with: :
        then type: string|int|bool|date|json
      */

      var type = /(?::(?:string|int|bool|date|json))?/; // example: {name_id:string} | {name-id} | {1name} | {_name-id}

      var identificator = new RegExp("{".concat(allowedString.source).concat(type.source, "}"));
      /* simple path OR dynamic path OR identificator
        Example: path | :id | {id:int}
      */

      var section = new RegExp("(".concat(allowedString.source, "|:").concat(allowedVariableName.source, ")|").concat(identificator.source)); // empty slash (/) OR (/section)+(/|urlParams)?
      // example: /name/:id/{number:int} | /add/ | /add?value1&value2 | /id?param | /

      var correctPath = new RegExp("^/|(?:(?:/".concat(section.source, ")+(/|").concat(urlParams.source, ")?)$"), 'g');
      return correctPath.test(path);
    };

    if (typeof path === 'string' && !checkRegisterPath(path)) {
      err.push('Incorrect path name');
    }

    if (err.length) {
      console.error('Route config for path "' + path + '" is invalid.');
      err.forEach(function (error) {
        console.error('Route config error: ' + error);
      });
    }

    return !!err.length;
  }

  function deregisterPath(stateName) {
    router.deregisterPath(stateName);
    delete routes[stateName];
  }

  function registerPath(path, config) {
    try {
      var fullPath = (config.parent ? config.parent : '') + path;
      if (validateRouteConfig(path, config)) return;

      if (Array.isArray(config.controller) && config.controller.length) {
        routeControllers[fullPath] = [];
        config.controller.forEach(function (_controller) {
          routeControllers[fullPath].push(new Promise(function (resolve) {
            runtime.get(_controller).then(function (TheController) {
              var controllerInstance = _.assign({}, new ControllerPrototype(), new TheController());

              resolve(controllerInstance);
            });
          }));
        });
      }

      var onBefore = function onBefore(cb, params) {
        inTransit = true;
        nextRoute = {
          config: config,
          controllers: routeControllers[fullPath],
          params: params
        };

        if (moderateNavigation(cb)) {
          if (currentRoute.config && !currentRoute.config.doNotUnloadOnBefore) {
            currentRoute.config.onunload();
          }

          var layout = runtime.getComponent('kl-layout');

          if (layout) {
            layout.setLayout(config.layout);
          }

          cb();
        }
      };

      var onActivate = function onActivate(params) {
        onNavigate(config, routeControllers[fullPath], params);
      };

      routes[config.stateName] = config;
      router.registerPath(path, config, onActivate, onBefore, function () {
        deactivate(routeControllers[fullPath], {}, config.onunload);
      });
    } catch (e) {
      logger.error('Unable to register route: ');
      logger.error(e);
    }
  }

  function connect(routerAdapter) {
    if (routerAdapter && typeof routerAdapter.registerPath === 'function') {
      router = routerAdapter;
    } else {
      throw new Error('This is not a Router Adapter');
    }
  }

  function getCurrentRoute() {
    return currentRoute;
  }

  function getNextRoute() {
    return nextRoute;
  }

  function getPrevRoute() {
    return prevRoute;
  }
  /**
  * @param {function} callbackFunc - behavior when navigation is disabled
  */


  function disableNavigation(callbackFunc) {
    isNavigationDisabled = true;
    onDisabledNavigation = callbackFunc;
  }

  function enableNavigation() {
    isNavigationDisabled = false;
    onDisabledNavigation = null;
  }
  /**
  * Check navigation oportunity
  * If negative result - apply rejectFunc and OnDisable functions
  * @param {function} reject
  * @return {boolean} - can navigate further
  */


  function moderateNavigation(rejectFunc) {
    if (isNavigationDisabled) {
      if (rejectFunc) {
        rejectFunc.redirect(Router.currentState.name, Router.currentState.parameters);
      }

      onDisabledNavigation();
    }

    return !isNavigationDisabled;
  }
  /**
  * Get route config by state name
  * @param {string} stateName
  * @return {Object} route config
  */


  function getRoute(stateName) {
    return routes[stateName];
  }

  function isAllowedToRegisterExtensionOnState(stateName) {
    var route = this.getRoute(stateName);
    return Boolean(route && route.allowExtensionRegister);
  }

  function getAllowedRouteStatesForExtension() {
    var reducer = function reducer(arr, currentValue) {
      if (currentValue.allowExtensionRegister) arr.push(currentValue.stateName);
      return arr;
    };

    return Object.values(routes).reduce(reducer, []);
  }

  var api = {
    connect: connect,
    registerPath: registerPath,
    deregisterPath: deregisterPath,
    getCurrentRoute: getCurrentRoute,
    getNextRoute: getNextRoute,
    getPrevRoute: getPrevRoute,
    getRoute: getRoute,
    disableNavigation: disableNavigation,
    enableNavigation: enableNavigation,
    moderateNavigation: moderateNavigation,
    isAllowedToRegisterExtensionOnState: isAllowedToRegisterExtensionOnState,
    getAllowedRouteStatesForExtension: getAllowedRouteStatesForExtension
  };
  return api;
};